import { notification } from 'antd'
import { createSearchParams, history } from '@umijs/max'
import RightContent from '@/components/RightContent'
import consts from './utils/consts'
import { queryCurrentUser } from './services/api/login'
import defaultSettings from '../config/defaultSettings'
import type { Settings as LayoutSettings } from '@ant-design/pro-layout'
import type { RequestConfig, RunTimeLayoutConfig } from '@umijs/max'

export async function getInitialState(): Promise<{
  settings?: Partial<LayoutSettings>
  currentUser?: API.CurrentUser
  fetchUserInfo?: () => Promise<API.CurrentUser | undefined>
}> {
  const fetchUserInfo = async () => {
    try {
      const msg = await queryCurrentUser()
      return msg.data
    } catch (error) {
      history.push(consts.loginPath)
    }
    return undefined
  }
  // 如果是登录页面，不执行
  if (history.location.pathname !== consts.loginPath) {
    const currentUser = await fetchUserInfo()
    return {
      fetchUserInfo,
      ...currentUser,
      settings: {}
    }
  }
  return {
    fetchUserInfo,
    settings: {}
  }
}

const authHeaderInterceptor = options => {
  const token = window.localStorage.getItem('TOKEN_ID')
  // 如果是登录页面，不执行
  if (token) {
    // 在白名单里的请求放过
    if (consts.TOKEN_WHITE_LIST.includes(options.url)) {
      return options
    }
    options.headers[consts.TOKEN_HEADER] = `bearer ${token}`
  }
  return options
}

const errorHandler = err => {
  if (err.name === 'BizError') {
    const response = err.info
    if (consts.TOKEN_INVALID_CODE.includes(response.code) && window.location.pathname !== consts.loginPath) {
      notification.error({
        message: '用户信息过期',
        description: '请重新登录'
      })
      history.replace({
        pathname: consts.loginPath,
        search: createSearchParams({
          redirect: window.location.pathname
        }).toString()
      })
      return
    }
    notification.error({
      message: '请求失败',
      description: response.msg
    })
    return
  }
  // 那么对于其他的错误, 需要实践中一个个找出来
  if (err.name === 'RequestError') {
    switch (err.type) {
      case 'Timeout': {
        message.error('系统超时,请联系开发人员')
      }
    }
    return
  }
}

const responseInterceptor = response => {
  const { code = -1 } = response?.data || {}
  if (code !== consts.RET_CODE.SUCCESS) {
    return Promise.reject(response?.data)
  }
  return Promise.resolve(response)
}
export const request: RequestConfig = {
  errorConfig: {
    errorThrower: res => {
      const { data, code, msg } = res
      if (code !== consts.RET_CODE.SUCCESS) {
        const error: any = new Error(errorMessage)
        error.name = 'BizError'
        error.info = { code, msg, data }
        throw error // 抛出自制的错误
      }
    },
    errorHandler
  },
  baseURL: consts.PREFIX_URL,
  responseInterceptors: [responseInterceptor],
  requestInterceptors: [authHeaderInterceptor]
}

export const layout: RunTimeLayoutConfig = ({ initialState }) => {
  return {
    rightContentRender: () => <RightContent />,
    disableContentMargin: false,
    onPageChange: () => {
      if (!initialState?.currentUser?.ID && window.location.pathname !== consts.loginPath) {
        history.replace({
          pathname: consts.loginPath,
          search: createSearchParams({
            redirect: history.location.pathname
          }).toString()
        })
      }
    },
    ...defaultSettings
  }
}
