import dayjs from 'dayjs'
import ProTable from '@ant-design/pro-table'
import consts from '@/utils/consts'
import { Button } from 'antd'
import { useMemo, useRef, useState } from 'react'
import { history, Link } from '@umijs/max'
import { queryInstitutionList } from '@/services/api/institution'
import type { ProColumnType, ActionType } from '@ant-design/pro-table'
import { PageContainer } from '@ant-design/pro-layout'
import CompanyDrawer from './components/CompanyDrawer'
import { ModalType } from '@/utils/enum'
import AnimateContent from '@/components/AnimateContent'
import type { ColumnsStateType } from '@ant-design/pro-table/lib/typing'

export type InstitutionDetailProps = {
  dataID: string
}

const CompanyList = () => {
  const tRef = useRef<ActionType>(null)
  const [state, setState] = useState({
    params: {
      search: null
    },
    visible: false,
    currentModalType: ModalType.ADD,
    defaultFormData: null
  })

  const columns: ProColumnType<API.InstitutionListItem>[] = [
    {
      dataIndex: 'name',
      title: '企事业单位名称',
      width: 286,
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      render: (name, record) => (
        <Link to={`/institutions/company/${record.ID}`} className="text-primary cursor-pointer">
          {name}
        </Link>
      )
    },
    {
      dataIndex: 'acronym',
      title: '企事业单位简称',
      width: 286,
      onHeaderCell: () => ({ style: { textAlign: 'center' } })
    },
    {
      dataIndex: 'institutionAcountTotal',
      title: '人员账号',
      renderText: (text, record) => `${text ?? 0}/${record.memberTotal}`,
      width: 96,
      onHeaderCell: () => ({ style: { textAlign: 'center' } })
    },
    {
      dataIndex: 'createdTime',
      title: '创建时间',
      width: 106,
      align: 'center',
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      renderText: text => dayjs(text).format('YYYY-MM-DD')
    },
    {
      dataIndex: 'created',
      title: '创建人',
      width: 116,
      align: 'center',
      onHeaderCell: () => ({ style: { textAlign: 'center' } })
    },
    {
      title: '操作',
      dataIndex: 'operate',
      width: 61,
      align: 'center',
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      render: (_, record) => (
        <div className="divide-x divide-bg-gray-400 flex flex-row justify-center">
          <div
            className="text-primary cursor-pointer hover:text-hex-967bbd"
            onClick={() => {
              setState({
                ...state,
                visible: true,
                currentModalType: ModalType.UPDATE,
                defaultFormData: record
              })
            }}>
            编辑
          </div>
        </div>
      )
    }
  ]

  const columnsState: ColumnsStateType = useMemo(
    () => ({
      persistenceKey: 'institution_list_column_key',
      persistenceType: 'localStorage'
    }),
    []
  )

  return (
    <PageContainer title={false}>
      <ProTable<API.InstitutionListItem>
        rowKey="ID"
        params={state.params}
        actionRef={tRef}
        columnsState={columnsState}
        scroll={{ y: document.body.clientHeight - 315 }}
        columns={columns}
        request={async (params, filter, sorter) => {
          const { code = -1, data: { items = [], total = 0 } = {} } = await queryInstitutionList({
            ...params,
            ...filter,
            ...sorter
          })
          return {
            data: items,
            success: code === consts.RET_CODE.SUCCESS,
            total
          }
        }}
        toolbar={{
          search: {
            onSearch: val => setState({ ...state, params: { ...state.params, search: val } }),
            style: { width: '250px' },
            placeholder: '请输入企事业单位名称'
          },
          actions: [
            <Button
              key="add"
              type="primary"
              onClick={() => setState({ ...state, visible: true, currentModalType: ModalType.ADD })}>
              新建企事业单位
            </Button>
          ]
        }}
        search={false}
      />
      <AnimateContent
        title={state.currentModalType === ModalType.ADD ? '新增企事业单位' : '编辑企事业单位'}
        visible={state.visible}
        onVisibleChange={visible => setState({ ...state, visible })}>
        <CompanyDrawer
          type={state.currentModalType}
          defaultFormData={state.defaultFormData}
          reload={() => tRef.current?.reload()}
          setVisible={(visible: boolean) => setState({ ...state, visible })}
        />
      </AnimateContent>
    </PageContainer>
  )
}

export default CompanyList
