import React, { useEffect, useMemo, useRef } from 'react'
import { message, Button } from 'antd'
import { connect, useRequest } from '@umijs/max'
import { addAccount, updateAccount } from '@/services/api/institution'
import { BaseMenuEnum } from '@/pages/Schema/Base'
import type { ConnectProps } from '@umijs/max'
import type { InstitutionsModelState } from '../../model'
import { ModalType } from '@/utils/enum'
import type { ProFormInstance } from '@ant-design/pro-form'
import { ModalForm, ProFormText } from '@ant-design/pro-form'
import { changeAccountPsw } from '@/services/api/user'
import { createForm, onFieldMount, onFieldReact } from '@formily/core'
import { createSchemaField } from '@formily/react'
import { loadOrganizationalStructureList } from '@/utils/schema'
import {
  FormItem,
  Input,
  Switch,
  FormGrid,
  Select,
  FormLayout,
  Form,
  TreeSelect,
  FormButtonGroup,
  Submit,
  Radio,
  Checkbox,
  NumberPicker,
  Password,
  DatePicker,
  TimePicker,
  Cascader
} from '@formily/antd'
import { fetchRoleList } from '@/services/permission'
import consts from '@/utils/consts'
import { RoleType } from '@/pages/Permission/Role/components/RoleLeftMenu'
type StaffModalProps = ConnectProps & {
  readOnly: boolean
  onVisibleChange: (visible: boolean) => void
  type: ModalType
  defaultFormData?: Nullable<API.AccountListItem>
  accountTypeList: API.AccountType
  organizationList: API.OrganizationalStructureListItem[]
  reload: () => void
  schema?: Record<string, any> | null
}

const StaffDrawer: React.FC<StaffModalProps> = ({
  onVisibleChange,
  staffSchema,
  dispatch,
  type,
  defaultFormData,
  accountTypeList,
  reload
}) => {
  const { institution, organizationalStructure, ...otherValues } = defaultFormData
  const formRef = useRef<ProFormInstance>(null)
  const { run: tryUpdateAccount } = useRequest(updateAccount, {
    manual: true,
    onSuccess: () => {
      message.success('更新成功')
    }
  })

  const { run: tryChangePsw } = useRequest(changeAccountPsw, {
    manual: true,
    onSuccess: () => {
      message.success('修改成功')
      // eslint-disable-next-line no-unused-expressions
      formRef.current?.setFieldsValue()
    }
  })

  const { run: tryAddAccount } = useRequest(addAccount, {
    manual: true,
    onSuccess: () => {
      message.success('创建成功')
    },
    throwOnError: true
  })

  useEffect(() => {
    if (!staffSchema) {
      dispatch({
        type: 'schemaBase/querySchema',
        payload: {
          columnType: BaseMenuEnum.STAFF
        }
      })
    }
  }, [])

  const onFinish = async formData => {
    if ('institution' in formData) delete formData.institution
    // 执行表单提交
    if (type === ModalType.ADD) {
      await tryAddAccount(formData)
    } else {
      await tryUpdateAccount({ ...formData, ID: defaultFormData.ID })
    }
    onVisibleChange(false)
    reload()
  }
  const normalForm = useMemo(
    () =>
      createForm({
        validateFirst: true,
        readPretty: type === ModalType.PREVIEW,
        initialValues: {
          ...otherValues,
          institutionID: institution.ID,
          organizationalStructureID: organizationalStructure?.ID
        },
        effects(form) {
          onFieldMount('password', field => {
            type !== ModalType.ADD && field.setDisplay('none')
          })
          onFieldMount('account', field => {
            type !== ModalType.ADD && (field.disabled = true)
          })
          onFieldMount('institutionID', field => {
            field.dataSource = [{ label: institution?.name, value: institution?.ID }]
          })
          onFieldMount('accountType', field => {
            field.dataSource = accountTypeList
          })
          onFieldReact('organizationalStructureID', async field => {
            if (type === ModalType.PREVIEW) {
              defaultFormData.organizationalStructure &&
                (field.dataSource = [
                  {
                    label: defaultFormData.organizationalStructure.name,
                    value: defaultFormData.organizationalStructure.ID
                  }
                ])
            } else {
              const institutionID = form.values.institutionID
              const dataSource = await loadOrganizationalStructureList({ dataID: institutionID })

              field.dataSource = dataSource
            }
          })
          onFieldMount('roleIDs', async field => {
            const { code = -1, data } = await fetchRoleList({ normal: true })
            if (code === consts.RET_CODE.SUCCESS) {
              field.dataSource = data.map(item => ({ label: item.name, value: item.ID }))
              if (type === ModalType.ADD && data?.[0]?.roleType === RoleType.SYSTEM_NORMAL) {
                field.value = data[0].ID
              }
            }
          })
        }
      }),
    []
  )
  const SchemaField = createSchemaField({
    components: {
      FormLayout,
      FormItem,
      FormGrid,
      Input,
      Switch,
      Select,
      TreeSelect,
      Radio,
      Checkbox,
      NumberPicker,
      Password,
      DatePicker,
      TimePicker,
      Cascader
    }
  })

  return (
    <Form form={normalForm} labelCol={6} wrapperCol={8} className="max-w-500px mt-6">
      <SchemaField schema={staffSchema} />
      <FormButtonGroup.Sticky>
        <FormButtonGroup.FormItem>
          {type === ModalType.UPDATE ? (
            <ModalForm
              formRef={formRef}
              title="修改密码"
              modalProps={{ width: '30vw' }}
              isKeyPressSubmit
              trigger={<Button>重置密码</Button>}
              onFinish={async values => {
                try {
                  await tryChangePsw({ ...values, ID: defaultFormData?.ID })
                  return true
                } catch (error) {
                  return false
                }
              }}>
              <ProFormText.Password
                label="新密码"
                name="newPassword"
                rules={[
                  { required: true, message: '请输入新密码' },
                  () => ({
                    validator(_, value) {
                      if (!value) {
                        return Promise.resolve()
                      }
                      if (!/(?=.*[a-zA-Z])(?=.*\d).{8,16}/.test(value)) {
                        return Promise.reject(new Error('密码长度最小8位且是数字和英文的组合'))
                      }
                      return Promise.resolve()
                    }
                  })
                ]}
              />
              <ProFormText.Password
                label="确定新密码"
                name="confirmPassword"
                rules={[
                  { required: true, message: '请输入新密码' },
                  ({ getFieldValue }) => ({
                    validator(_, value) {
                      if (!value || getFieldValue('newPassword') === value) {
                        return Promise.resolve()
                      }
                      return Promise.reject(new Error('两次输入的密码不一致!'))
                    }
                  })
                ]}
              />
            </ModalForm>
          ) : null}
          {type !== ModalType.PREVIEW ? <Submit onSubmit={onFinish}>提交</Submit> : null}
        </FormButtonGroup.FormItem>
      </FormButtonGroup.Sticky>
    </Form>
  )
}

export default connect(
  ({
    institutions,
    schemaBase
  }: {
    institutions: InstitutionsModelState
    schemaBase: SchemaBaseModelState
  }) => ({
    accountTypeList: institutions.accountType.map(item => ({
      label: item.name,
      value: item.value
    })),
    // organizationList: institutions.organizationType,
    staffSchema: schemaBase.base[BaseMenuEnum.STAFF]
  })
)(StaffDrawer)
