import ProTable from '@ant-design/pro-table'
import type { ProColumnType, ActionType } from '@ant-design/pro-table'
import { PageContainer } from '@ant-design/pro-layout'
import consts from '@/utils/consts'
import { useMemo, useRef, useState } from 'react'
import type { ConnectProps } from '@umijs/max'
import { queryAccountList } from '@/services/api/institution'
import dayjs from 'dayjs'
import StaffDetail from './components/StaffDetail'
import AnimateContent from '@/components/AnimateContent'
import { ModalType } from '@/utils/enum'
import type { ColumnsStateType } from '@ant-design/pro-table/lib/typing'

type ListProps = ConnectProps & {
  accountTypeList: { label: string; value: string }[]
}

export const genderEum = {
  0: '男',
  1: '女'
}

const CompanyList: React.FC<ListProps> = () => {
  const tRef = useRef<ActionType>(null)
  const [state, setState] = useState({
    params: {
      search: null,
      institutionID: null
    },
    title: '',
    visible: false,
    currentModalType: ModalType.ADD,
    defaultFormData: {}
  })

  const columns: ProColumnType<API.AccountListItem>[] = [
    {
      dataIndex: 'account',
      key: 'account',
      title: '账号',
      width: 116,
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      render: (name, record) => (
        <div
          className="text-primary cursor-pointer hover:text-hex-967bbd"
          onClick={() => {
            setState({
              ...state,
              visible: true,
              currentModalType: ModalType.PREVIEW,
              title: record.name,
              defaultFormData: record
            })
          }}>
          {name}
        </div>
      )
    },
    {
      dataIndex: 'name',
      key: 'name',
      title: '名称',
      width: 86,
      onHeaderCell: () => ({ style: { textAlign: 'center' } })
    },
    {
      dataIndex: 'enable',
      key: 'enable',
      title: '状态',
      width: 66,
      align: 'center',
      filters: true,
      filterMultiple: false,
      defaultFilteredValue: ['true'],
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      valueEnum: {
        true: { text: '启用', status: 'Success' },
        false: { text: '禁用', status: 'Error' }
      }
    },
    {
      dataIndex: 'position',
      key: 'position',
      title: '职位',
      align: 'cnenter',
      width: 120,
      ellipsis: true,
      onHeaderCell: () => ({ style: { textAlign: 'center' } })
    },
    {
      dataIndex: 'organizationalStructureID',
      key: 'organizationalStructureID',
      title: '组织架构',
      width: 146,
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      renderText: (_, record) => record.organizationalStructure?.name
    },
    {
      dataIndex: 'institutionID',
      key: 'institutionID',
      title: '所属企事业单位',
      width: 286,
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      renderText: (_, record) => record.institution.name
    },
    {
      dataIndex: 'contactMobile',
      key: 'contactMobile',
      title: '联系手机',
      width: 116,
      align: 'center',
      onHeaderCell: () => ({ style: { textAlign: 'center' } })
    },
    {
      dataIndex: 'officePhone',
      key: 'officePhone',
      title: '办公电话',
      width: 136,
      align: 'center',
      onHeaderCell: () => ({ style: { textAlign: 'center' } })
    },
    {
      dataIndex: 'createdTime',
      key: 'createdTime',
      width: 106,
      align: 'center',
      title: '创建时间',
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      renderText: text => dayjs(text).format('YYYY-MM-DD')
    },
    {
      dataIndex: 'created',
      key: 'created',
      title: '创建者',
      width: 116,
      align: 'center',
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      renderText: (_, record) => record?.created?.name
    },
    {
      title: '操作',
      dataIndex: 'operate',
      width: 61,
      align: 'center',
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      render: (_, record) => (
        <div className="divide-x divide-bg-gray-400 flex flex-row justify-center">
          <div
            className="text-primary cursor-pointer hover:text-hex-967bbd"
            onClick={() => {
              // console.log(record)
              setState({
                ...state,
                visible: true,
                currentModalType: ModalType.UPDATE,
                title: record.name,
                defaultFormData: record
              })
            }}>
            编辑
          </div>
        </div>
      )
    }
  ]

  const columnsState: ColumnsStateType = useMemo(
    () => ({
      persistenceKey: 'staff_list_column_key',
      persistenceType: 'localStorage'
    }),
    []
  )

  return (
    <PageContainer title={false}>
      <ProTable<API.AccountListItem>
        rowKey="ID"
        params={state.params}
        actionRef={tRef}
        columnsState={columnsState}
        scroll={{ y: document.body.clientHeight - 315 }}
        columns={columns}
        request={async (params, filter, sorter) => {
          const { code = -1, data: { items = [], total = 0 } = {} } = await queryAccountList({
            ...params,
            ...filter,
            ...sorter
          })
          return {
            data: items,
            success: code === consts.RET_CODE.SUCCESS,
            total
          }
        }}
        toolbar={{
          search: {
            onSearch: val => setState({ ...state, params: { ...state.params, search: val } }),
            style: { width: '250px' },
            placeholder: '请输入账号、姓名、企事业单位'
          }
        }}
        search={false}
      />
      <AnimateContent
        title={
          <>
            {state.currentModalType === ModalType.UPDATE ? `编辑账号(${state.title})` : null}
            {state.currentModalType === ModalType.PREVIEW ? `员工详情(${state.title})` : null}
          </>
        }
        visible={state.visible}
        onVisibleChange={visible => setState({ ...state, visible })}>
        <StaffDetail
          type={state.currentModalType}
          defaultFormData={state.defaultFormData}
          visible={state.visible}
          reload={() => tRef.current?.reload()}
          onVisibleChange={(visible: boolean) => setState({ ...state, visible })}
        />
      </AnimateContent>
    </PageContainer>
  )
}

// export default connect(({ institutions }: { institutions: InstitutionsModelState }) => ({
//   accountTypeList: institutions.accountType.map(item => ({
//     label: item.name,
//     value: item.value
//   }))
// }))(CompanyList)
export default CompanyList
