import { delProject, getProjectList } from '@/services/api/project'
import ProTable from '@ant-design/pro-table'
import { Tag, Modal } from 'antd'
import consts from '@/utils/consts'
import { useRef, useState, useEffect, useMemo } from 'react'
import { connect, history, Link, useRequest } from '@umijs/max'
import { DeleteOutlined } from '@ant-design/icons'
import ProjectModal, { ModalType } from './components/ProjectModal'
import dayjs from 'dayjs'
import { PageContainer } from '@ant-design/pro-layout'
import { BaseMenuEnum } from '@/pages/Schema/Base'
import AnimateContent from '@/components/AnimateContent'
import type { ProColumnType, ActionType } from '@ant-design/pro-table'
import type { ConnectProps } from '@umijs/max'
import type { SchemaBaseModelState } from '@/pages/Schema/Base/model'
import type { ColumnsStateType } from '@ant-design/pro-table/lib/typing'

type ListProps = ConnectProps & {
  pTypeList: { label: string; value: string }[]
  schema?: Record<string, any> | null
}

const List: React.FC<ListProps> = ({ schema, dispatch, pTypeList }) => {
  const tRef = useRef<ActionType>(null)
  useEffect(() => {
    if (!schema) {
      dispatch({
        type: 'schemaBase/querySchema',
        payload: {
          columnType: BaseMenuEnum.PROJECT
        }
      })
    }
  }, [])

  const [state, setState] = useState({
    params: {
      search: null
    },
    title: '',
    visible: false,
    currentModalType: ModalType.ADD,
    defaultFormData: null
  })
  const { run: tryDelProject } = useRequest(delProject, {
    manual: true,
    onSuccess: () => tRef.current?.reload()
  })

  const goToDetail = params => {
    return history.push({
      pathname: '/project/management/detail',
      state: params
    })
  }

  const handleDelProject = (ID: string) => {
    Modal.confirm({
      title: '删除项目',
      content: (
        <div>
          <div>确认删除该项目？</div>
          <div>删除后，数据无法恢复，请谨慎操作。</div>
        </div>
      ),
      okButtonProps: {
        danger: true
      },
      onOk: () => tryDelProject({ ID })
    })
  }

  const columns: ProColumnType<API.ProjectListItem>[] = [
    {
      dataIndex: 'name',
      title: '项目名称',
      width: 286,
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      render: (text, record) => (
        <Link
          to={`/project/management/${record.ID}`}
          className="text-primary cursor-pointer hover:text-hex-967bbd">
          {text}
        </Link>
      )
    },
    {
      dataIndex: 'createdTime',
      title: '创建时间',
      width: 116,
      align: 'center',
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      renderText: text => dayjs(text).format('YYYY-MM-DD')
    },
    {
      dataIndex: 'createdID',
      key: 'createdID',
      title: '创建人',
      width: 326,
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      renderText: (_, record) =>
        record?.created?.ID ? `${record?.created?.name}（${record?.created?.institution?.name}）` : null
    },
    {
      dataIndex: 'reportAccount',
      key: 'reportAccount',
      title: '编辑者',
      width: 256,
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      renderText: (_, record) => (
        <div>
          {record.reportAccount?.map(item => (
            <Tag key={item.ID}>{item.name}</Tag>
          ))}
        </div>
      )
    },
    {
      title: '操作',
      dataIndex: 'operate',
      width: 91,
      align: 'center',
      onHeaderCell: () => ({ style: { textAlign: 'center' } }),
      render: (_, record) => (
        <div className="divide-x divide-bg-gray-400 flex flex-row justify-center items-center">
          {/* <div
            className="pr-2 text-primary cursor-pointer hover:text-hex-967bbd"
            onClick={() => {
              setState({
                ...state,
                visible: true,
                currentModalType: ModalType.UPDATE,
                title: record.name,
                defaultFormData: record
              })
            }}>
            编辑
          </div> */}

          <div
            className="text-hex-fd3995 cursor-pointer hover:text-hex-e7026e"
            onClick={() => handleDelProject(record.ID)}>
            <DeleteOutlined />
          </div>
        </div>
      )
    }
  ]

  const columnsState: ColumnsStateType = useMemo(
    () => ({
      persistenceKey: 'project_list_column_key',
      persistenceType: 'localStorage'
    }),
    []
  )

  const onAnimateChange = visible => setState({ ...state, visible })
  return (
    <PageContainer title={false}>
      <ProTable<API.ProjectListItem>
        rowKey="ID"
        params={state.params}
        actionRef={tRef}
        columnsState={columnsState}
        scroll={{ y: document.body.clientHeight - 315 }}
        columns={columns}
        request={async (params, filter, sorter) => {
          const { code = -1, data: { items = [], total = 0 } = {} } = await getProjectList({
            ...params,
            ...filter,
            ...sorter
          })
          return {
            data: items,
            success: code === consts.RET_CODE.SUCCESS,
            total
          }
        }}
        toolbar={{
          search: {
            onSearch: val => setState({ ...state, params: { ...state.params, search: val } }),
            style: { width: '250px' },
            placeholder: '请输入项目名称'
          }
          // actions: [
          //   <Button
          //     key="add"
          //     type="primary"
          //     onClick={() =>
          //       setState({
          //         ...state,
          //         visible: true,
          //         currentModalType: ModalType.ADD,
          //         title: null
          //       })
          //     }>
          //     新建项目
          //   </Button>
          // ]
        }}
        search={false}
      />

      <AnimateContent
        title={
          <>
            {state.currentModalType === ModalType.ADD ? '新增项目' : null}
            {state.currentModalType === ModalType.UPDATE ? '编辑-' : null}
            {state.title}
          </>
        }
        visible={state.visible}
        onVisibleChange={onAnimateChange}>
        <ProjectModal
          type={state.currentModalType}
          defaultFormData={state.defaultFormData}
          pTypeList={pTypeList}
          visible={state.visible}
          reload={() => tRef.current?.reload()}
          setVisible={onAnimateChange}
        />
      </AnimateContent>
    </PageContainer>
  )
}

export default connect(({ schemaBase }: { schemaBase: SchemaBaseModelState }) => ({
  schema: schemaBase.base[BaseMenuEnum.PROJECT]
}))(List)
