import React, { useContext, useEffect, useMemo, useState } from 'react'
import { ApprovalWay, approvalWayConfigurationsEnum, ParticipantMode } from '../../enum'
import { ConfigureType } from '../../enum'
import { queryAccountList } from '@/services/api/institution'
import { Col, Collapse, Row, Select, Switch } from 'antd'
import { useRequest } from '@umijs/max'
import { eId, FlowContext } from '../../context'
import { isEdge, isNode } from 'react-flow-renderer'

type ParticipantCardProps = {
  approvalWay: ApprovalWay
  defaultData: {
    participantMode: ParticipantMode
    ID: string | null
    name: string | null
    institutionID: string | null
    configure: ConfigureType[]
  }
  institutionList: { label: string; value: string }[]
  updateAction: (values: Recordable<string>) => void
}

const configureItemTitle = {
  [ConfigureType.ADDSIGN]: {
    name: '允许加签',
    short: '签'
  },
  [ConfigureType.ASSISTAUDIT]: {
    name: '允许协审',
    short: '协'
  },
  [ConfigureType.RETURN]: {
    name: '允许退回',
    short: '退'
  },
  [ConfigureType.REVOKE]: {
    name: '允许撤回',
    short: '撤'
  },
  [ConfigureType.SKIP]: {
    name: '允许跳过',
    short: '跳'
  },
  [ConfigureType.NEXTSECTOR]: {
    name: '允许添加下一环节',
    short: '环'
  }
}

const ParticipantCard: React.FC<ParticipantCardProps> = ({
  approvalWay,
  updateAction,
  defaultData,
  institutionList
}) => {
  const { flowState } = useContext(FlowContext)
  const { modalConfig: { nodeID = null } = {}, elements } = flowState
  const curNode = elements.filter(item => isNode(item)).find(item => item.id === nodeID)
  const lastEdge = elements.filter(item => isEdge(item)).find(item => item.target === eId)

  const [state, setState] = useState({
    configureArr: approvalWayConfigurationsEnum[approvalWay],
    staffList: []
  })
  const { run: tryUpdateStaffList } = useRequest(
    dataID => queryAccountList({ current: 1, pageSize: 214000, dataID, enable: ['true'] }),
    {
      manual: true,
      onSuccess(result) {
        setState({
          ...state,
          staffList: result.items.map(item => ({ label: item.name, value: item.ID }))
        })
      }
    }
  )

  useEffect(() => {
    if (defaultData.ID && defaultData.institutionID && !state.staffList?.length) {
      tryUpdateStaffList(defaultData.institutionID)
    }
  }, [])

  useEffect(() => {
    // 指定用户模式下，加上下一个环节的配置信息
    if (
      approvalWay === ApprovalWay.ACCOUNT &&
      [ParticipantMode.ACCOUNT].includes(defaultData.participantMode)
    ) {
      // 当前的node元素
      // 最后一个edge及诶单
      if (lastEdge && curNode && lastEdge.source === curNode.id) {
        setState({ ...state, configureArr: [...state.configureArr, ConfigureType.NEXTSECTOR] })
      }
    }
  }, [defaultData.participantMode, curNode, lastEdge])
  if (defaultData.ID && !state.staffList?.length) return null
  return (
    <>
      <div className="children:mb-4">
        <div>
          <Select
            className="w-full"
            defaultValue={defaultData.participantMode}
            options={[
              { label: '仅用户模式', value: ParticipantMode.ACCOUNT }
              // { label: '角色模式', value: ParticipantMode.ROLE }
            ]}
            onChange={value => {
              updateAction({ ...defaultData, participantMode: value })
            }}
          />
        </div>
        <div>
          <Select
            className="w-full"
            defaultValue={defaultData.institutionID}
            options={institutionList}
            onChange={value => {
              if (value) tryUpdateStaffList(value)
              updateAction({ ...defaultData, institutionID: value })
            }}
          />
        </div>
        <div>
          <Select
            className="w-full"
            defaultValue={defaultData.ID}
            options={state.staffList}
            onChange={(_, options) => {
              updateAction({ ...defaultData, ID: options?.value, name: options?.label })
            }}
          />
        </div>
      </div>
      <Collapse
        defaultActiveKey={['1']}
        ghost
        expandIcon={({ isActive }) => (
          <span className="text-primary font-medium">{isActive ? '收叠' : '展开'}</span>
        )}
        expandIconPosition="right">
        <Collapse.Panel
          header={
            <div className="flex flex-row justify-between items-center">
              <div className="text-md font-midium h-24px leading-24px">配置信息</div>
              <div className="children:mx-1 flex flex-row ml-1">
                {defaultData.configure.map(item => (
                  <div
                    key={item}
                    className="w-30px text-center rounded-lg text-primary border border-hex-0089ff bg-hex-e9f5ff">
                    {configureItemTitle[item]?.short}
                  </div>
                ))}
              </div>
            </div>
          }
          key="1">
          <Row>
            {state.configureArr.map(item => (
              <Col span={8} key={item}>
                <div className="py-2 flex flex-row items-center ">
                  <div className="mr-2">{configureItemTitle[item]?.name}</div>
                  <Switch
                    checked={defaultData.configure.includes(item)}
                    onChange={checked => {
                      updateAction({
                        ...defaultData,
                        configure: checked
                          ? [...defaultData.configure, item]
                          : defaultData.configure.filter(conf => conf !== item)
                      })
                    }}
                  />
                </div>
              </Col>
            ))}
          </Row>
        </Collapse.Panel>
      </Collapse>
    </>
  )
}

export default ParticipantCard
