import { useState, useEffect, useContext, useMemo } from 'react'
import { Button, Drawer, message, Radio, Select, Tabs, Tooltip } from 'antd'
import { Actions, FlowContext } from '../../context'
import type { RadioChangeEvent } from 'antd'
import {
  ApprovalWay,
  approvalWayConfigurationsEnum,
  ConfigureType,
  ParticipantMode,
  VoteOption
} from '../../enum'
import { QuestionCircleFilled } from '@ant-design/icons'
import { queryInstitutionList } from '@/services/api/institution'
import { buildUUID } from '@/utils/uuid'
import styles from './index.less'
import ParticipantCard from './ParticipantCard'
import { useRequest } from '@umijs/max'
import { saveApprovalParticipant } from '@/services/api/project'
import consts from '@/utils/consts'
import classNames from 'classnames'

const FlowDrawer = () => {
  const { flowState, dispatch } = useContext(FlowContext)
  const { modalConfig: { nodeID = null, visible = false } = {}, dataID } = flowState
  const [state, setState] = useState({
    loading: false,
    institutionList: [],
    approvalWay: ApprovalWay.ACCOUNT,
    // voteWay: null, // 投票方式，仅会签模式下
    accounts: [],
    validityIDs: []
  })

  useEffect(() => {
    if (nodeID && visible) {
      const defaultConfig = flowState.flowData.get(nodeID) || {
        approvalWay: ApprovalWay.ACCOUNT,
        accounts: []
      }
      setState({ ...state, ...defaultConfig })
    }
  }, [nodeID])

  useRequest(() => queryInstitutionList({ current: 1, pageSize: 214000 }), {
    onSuccess(result) {
      setState({
        ...state,
        institutionList: result.items.map(item => ({ label: item.name, value: item.ID }))
      })
    }
  })

  // 会签、或签按钮不禁用
  const btnStatus = useMemo(() => {
    if (state.accounts.length >= 1) {
      return ![ApprovalWay.JOINTLYSIGN, ApprovalWay.ORSIGN].includes(state.approvalWay)
    }
    return false
  }, [state.approvalWay, state.accounts])

  const handleOnClose = () => dispatch({ type: Actions.CLOSE_MODAL })

  const plainOptions = [
    { label: '指定用户', value: ApprovalWay.ACCOUNT },
    { label: '会签', value: ApprovalWay.JOINTLYSIGN }
  ]

  // const voteOptions = [
  // { label: '全票通过', value: VoteOption.ALLVOTE }
  // { label: '一票否决', value: 'oneVoteNo' },
  // { label: '比例投票', value: 'percentageVote' },
  // { label: '人数投票', value: 'numVote' }
  // ]

  const approvalWayChange = (e: RadioChangeEvent) => {
    const { target } = e
    if (target?.value) {
      setState({ ...state, approvalWay: target.value, accounts: [] })
    }
  }

  const handleOnOk = async () => {
    setState({ ...state, loading: true })
    const { approvalWay, accounts } = state
    if (!accounts.every(item => item.institutionID && item.ID)) {
      setState({
        ...state,
        validityIDs: accounts.reduce((prev, curr) => {
          if (!curr.institutionID || !curr.ID) {
            prev.push(curr.uid)
          }
          return prev
        }, [])
      })
      return
    }
    const payload = {
      id: nodeID,
      node: {
        approvalWay,
        accounts
      }
    }
    const participantInfo: API.ParticipantInfo = {
      approvalWay: state.approvalWay,
      accounts: state.accounts
    }
    // if (state.approvalWay === ApprovalWay.JOINTLYSIGN) {
    //   participantInfo.voteWay = state.voteWay
    //   payload.node.voteWay = state.voteWay
    // }
    try {
      const { code = -1 } = await saveApprovalParticipant({
        ID: dataID,
        segmentID: nodeID,
        participantInfo
      })
      if (code === consts.RET_CODE.SUCCESS) {
        message.success('保存成功')
        setTimeout(() => {
          dispatch({
            type: Actions.SET_FLOW_NODE,
            payload
          })
        }, 80)
        handleOnClose()
      }
    } finally {
      setState({ ...state, loading: false })
    }
  }

  // 添加用户-参与者卡片
  const addParticipantInfo = () => {
    setState({
      ...state,
      accounts: [
        ...state.accounts,
        {
          uid: buildUUID(),
          participantMode: ParticipantMode.ACCOUNT,
          ID: null,
          name: null,
          institutionID: null,
          configure: approvalWayConfigurationsEnum[state.approvalWay]
        }
      ]
    })
  }

  const saveCardData = (targetID, formData) => {
    const validityIDs =
      formData.ID && formData.institutionID
        ? state.validityIDs.filter(ID => ID !== targetID)
        : state.validityIDs
    setState({
      ...state,
      accounts: state.accounts.map(item => {
        if (item.uid === targetID) {
          const newItem = { ...item, ...formData }
          return newItem
        }
        return item
      }),
      validityIDs
    })
  }

  const renderCard = () => {
    return state.accounts.map(participant => {
      if (!participant) return null

      const delCardItem = id => {
        setState({ ...state, accounts: state.accounts.filter(item => item.uid !== id) })
      }
      return (
        <div
          className={classNames(
            'shadow-base border rounded-md flex flex-col justify-between px-4 py-2 mt-6',
            styles.participantCard,
            { 'border-red-500': state.validityIDs.includes(participant.uid) }
          )}
          key={participant.uid}>
          <div className="flex flex-row justify-between">
            <div className="text-base font-medium">参与者</div>
            <div className="text-primary cursor-pointer" onClick={() => delCardItem(participant.uid)}>
              删除
            </div>
          </div>
          <div className="mt-3">
            <ParticipantCard
              approvalWay={state.approvalWay}
              institutionList={state.institutionList}
              defaultData={participant}
              updateAction={formData => saveCardData(participant.uid, formData)}
            />
          </div>
        </div>
      )
    })
  }

  return (
    <Drawer
      visible={visible}
      onClose={handleOnClose}
      destroyOnClose
      footer={
        <div className="flex justify-end">
          <Button onClick={handleOnClose}>取消</Button>
          <Button
            className="ml-8px"
            type="primary"
            loading={state.loading}
            disabled={!state.accounts.length}
            onClick={handleOnOk}>
            确定
          </Button>
        </div>
      }
      title={
        <div className="flex items-center">
          <span className="font-medium">设置审批人</span>
          <span className="ml-2 cursor-pointer">
            <Tooltip
              title={
                <div>
                  <div>指定用户：选择指定的用户或者角色完成审批。</div>
                  <div>
                    会签：指同一个审批节点设置多个人，如ABC三人，三人会同时收到审批，需全部同意之后，审批才可到下一审批节点
                  </div>
                  {/* <div>
                    或签：指同一个审批节点设置多个人，如ABC三人，三人会同时收到审批，只要其中任意一人审批即可到下一审批节点
                  </div>
                  <div>依次审批：按照顺序逐个审批</div>
                  <div>由上一次审批人设置：由上环节审批人设置下环节流程。</p> */}
                </div>
              }>
              <QuestionCircleFilled style={{ color: '#FEA100' }} />
            </Tooltip>
          </span>
        </div>
      }
      width="30%">
      <Tabs>
        <Tabs.TabPane tab="设置审批人" key="1">
          <div>
            <Radio.Group options={plainOptions} onChange={approvalWayChange} value={state.approvalWay} />
            {/* {state.approvalWay === ApprovalWay.JOINTLYSIGN ? (
              <div className="mt-20px">
                <Select
                  defaultValue={VoteOption.ALLVOTE}
                  options={voteOptions}
                  style={{ width: '100%' }}
                  onChange={value => setState({ ...state, voteWay: value })}
                />
              </div>
            ) : null} */}
          </div>
          <div className="mt-20px">
            <Button type="primary" disabled={btnStatus} onClick={() => addParticipantInfo()}>
              添加用户
            </Button>
          </div>
          {renderCard()}
        </Tabs.TabPane>
      </Tabs>
    </Drawer>
  )
}

export default FlowDrawer
