import { buildUUID } from '@/utils/uuid'
import { SolutionOutlined } from '@ant-design/icons'
import { Button, Popover } from 'antd'
import React, { useMemo, useState, useContext } from 'react'
import type { EdgeProps } from 'react-flow-renderer'
import { isNode } from 'react-flow-renderer'
import { getBezierPath, getEdgeCenter, getMarkerEnd, useStoreState } from 'react-flow-renderer'
import { Actions, eId, FlowContext } from '../../context'
import { generateElements, genreateElementEnum, getEdgeParams } from '../../utils'
import { addApprovalNode } from '@/services/api/project'
import styles from './index.less'
import 'antd/lib/button/style/css'
import consts from '@/utils/consts'
import useLoading from '../../hooks/useLoading'
import IconFont from '@/components/IconFont'
import { ConfigureType } from '../../enum'
const foreignObjectSize = 50

export enum SectorType {
  APPROVAL = 'approval',
  CONDITION = 'condition'
}

export function CommonEdge(props: EdgeProps) {
  const {
    id,
    sourceX,
    sourceY,
    targetX,
    targetY,
    source,
    target,
    style = {},
    arrowHeadType,
    markerEndId
  } = props

  const [visible, setVisible] = useState(false)
  const { flowState, dispatch } = useContext(FlowContext)
  const { elements, flowInstance, flowData, dataID, readPretty } = flowState
  const { run: showLoading } = useLoading()
  const setElements = els => {
    dispatch({
      type: Actions.SET_ELEMENTS,
      payload: els
    })
  }
  const showBtn =
    flowData.get(source) && target === eId
      ? flowData
          .get(source)
          ?.accounts.every(item => !item.configure.includes(ConfigureType.NEXTSECTOR))
      : true

  const nodes = useStoreState(store => store.nodes)
  const markerEnd = getMarkerEnd(arrowHeadType, markerEndId)

  const sourceNode = useMemo(() => nodes.find(n => n.id === source), [source, nodes])
  const targetNode = useMemo(() => nodes.find(n => n.id === target), [target, nodes])

  if (!sourceNode || !targetNode) {
    return null
  }

  const { sx, sy, tx, ty, sourcePos, targetPos } = getEdgeParams(sourceNode, targetNode)

  const d = getBezierPath({
    sourceX: sx,
    sourceY: sy - 2,
    sourcePosition: sourcePos,
    targetPosition: targetPos,
    targetX: tx,
    targetY: ty
  })
  const [edgeCenterX, edgeCenterY] = getEdgeCenter({
    sourceX,
    sourceY,
    targetX,
    targetY
  })

  const togglePopver = (isShow: boolean) => {
    setVisible(isShow)
  }

  const onEdgeClick = () => {
    togglePopver(true)
    // evt.stopPropagation()
    // alert(`remove ${id}`)
  }

  const addAuditor = async () => {
    const uid = buildUUID()
    const newNode = {
      id: uid,
      type: 'input',
      data: {}
    }
    const currentEdge = elements.find(item => item.id === id)
    const newElements = generateElements(genreateElementEnum.ADD, elements)(newNode, currentEdge)

    const { code = -1 } = await addApprovalNode({
      ID: dataID,
      segmentBrotherID: source,
      flowProcess: newElements,
      segment: {
        ID: uid,
        name: '审批人',
        sectorType: SectorType.APPROVAL
      }
    })
    if (code === consts.RET_CODE.SUCCESS) {
      setElements(newElements)
      dispatch({
        type: Actions.SET_FLOW_NODE,
        payload: {
          id: uid,
          node: null
        }
      })
      togglePopver(false)
      setTimeout(() => {
        showLoading()
        flowInstance?.fitView()
      }, 80)
    }
  }

  return (
    <>
      <path id={id} style={style} className={styles.flowPath} d={d} markerEnd={markerEnd} />
      {showBtn && !readPretty && (
        <foreignObject
          width={foreignObjectSize}
          height={foreignObjectSize}
          x={edgeCenterX - foreignObjectSize / 2}
          y={edgeCenterY - foreignObjectSize / 2}>
          <div className={styles.addIcon}>
            <Popover
              content={
                <ul className="m-0 p-0">
                  <li>
                    <Button type="dashed" onClick={addAuditor} icon={<SolutionOutlined />}>
                      审批人
                    </Button>
                  </li>
                  {/* <li className="mt-2 condition">
        <Button type="dashed" onClick={addAuditor} icon={<BranchesOutlined />}>
          条件分支
        </Button>
      </li> */}
                </ul>
              }
              trigger="click"
              placement="right"
              visible={visible}
              destroyTooltipOnHide
              onVisibleChange={togglePopver}
              overlayClassName="flow-popover">
              <Button
                icon={<IconFont type="icon-plus" />}
                shape="circle"
                size="small"
                onClick={event => onEdgeClick(event, id)}
              />
            </Popover>
          </div>
        </foreignObject>
      )}
    </>
  )
}
