import { getAccountTypeList, queryOrganizationalStructureList } from '@/services/api/institution'
import { getProjectTypeList } from '@/services/api/project'
import { action } from '@formily/reactive'
import consts from './consts'
import type { BaseMenuEnum } from '@/pages/Schema/Base'
import { SchemaEnum } from '@/pages/Schema/Base'
import type { ISchema } from '@formily/react'

// 项目schema
export const projectSchema = {
  type: 'object',
  properties: {
    name: {
      type: 'string',
      title: '名称',
      required: true,
      'x-decorator': 'FormItem',
      'x-component': 'Input',
      'x-index': 1
    },
    shortName: {
      type: 'string',
      title: '项目简称',
      'x-decorator': 'FormItem',
      'x-component': 'Input',
      'x-index': 2
    }
  }
}

// 企事业单位
export const institutionSchema = {
  type: 'object',
  properties: {
    name: {
      type: 'string',
      title: '企事业单位名称',
      required: true,
      'x-decorator': 'FormItem',
      'x-component': 'Input'
    },
    acronym: {
      type: 'string',
      title: '企事业单位简称',
      'x-decorator': 'FormItem',
      'x-component': 'Input'
    }
  }
}

// 人员信息
export const staffSchema = {
  type: 'object',
  properties: {
    enable: {
      type: 'boolean',
      title: '账号启用',
      required: true,
      'x-decorator': 'FormItem',
      'x-component': 'Switch',
      'x-index': 0
    },
    projectLimit: {
      type: 'number',
      required: true,
      title: '创建数量',
      default: 3,
      'x-decorator': 'FormItem',
      'x-component': 'NumberPicker',
      'x-index': 2
    },
    institutionID: {
      type: 'string',
      title: '事业单位',
      required: true,
      'x-decorator': 'FormItem',
      'x-component': 'Select',
      'x-disabled': true,
      'x-index': 3
    },
    account: {
      type: 'string',
      title: '账号',
      required: true,
      'x-decorator': 'FormItem',
      'x-component': 'Input',
      'x-index': 4
    },
    name: {
      type: 'string',
      title: '姓名',
      required: true,
      'x-decorator': 'FormItem',
      'x-component': 'Input',
      'x-index': 5
    },
    password: {
      type: 'string',
      title: '密码',
      required: true,
      'x-decorator': 'FormItem',
      'x-component': 'Input',
      'x-index': 6
    },
    roleIDs: {
      type: 'string',
      title: '角色',
      required: true,
      'x-decorator': 'FormItem',
      'x-component': 'Select',
      'x-component-props': {
        mode: 'multiple'
      },
      'x-index': 7
    },
    organizationalStructureID: {
      type: 'string',
      title: '组织架构',
      'x-decorator': 'FormItem',
      'x-component': 'TreeSelect',
      'x-index': 8
    },
    position: {
      type: 'string',
      title: '职位',
      'x-decorator': 'FormItem',
      'x-component': 'Input',
      'x-index': 9
    },
    contactMobile: {
      type: 'string',
      title: '联系手机',
      'x-decorator': 'FormItem',
      'x-component': 'Input',
      'x-index': 10,
      'x-validator': [
        {
          pattern: '^(?:(?:\\+|00)86)?1\\d{10}$',
          message: '请输入正确的手机号码',
          whitespace: true
        }
      ]
    },
    officePhone: {
      type: 'string',
      title: '办公电话',
      'x-decorator': 'FormItem',
      'x-component': 'Input',
      'x-index': 11,
      'x-validator': [
        {
          pattern: '^(?:(?:\\d{3}-)?\\d{8}|^(?:\\d{4}-)?\\d{7,8})(?:-\\d+)?$',
          whitespace: true,
          message: '请输入正确的办公电话号码'
        }
      ]
    }
  }
}

/** 组合schema */
export const connectSchema = (initialSchemas, schemas) => {
  if (!schemas) return initialSchemas
  const startIdx = Object.keys(initialSchemas.properties).length
  let i = 0
  const b = { ...initialSchemas }
  for (const key in schemas) {
    if (Object.prototype.hasOwnProperty.call(schemas, key)) {
      schemas[key]['x-index'] = startIdx + i
      i++
    }
  }
  b.properties = { ...b.properties, ...schemas }
  console.log(b)

  return b
}

/**
 * 根据传入的数据格式化成表单所需的数据
 * */
export const transformToSchemaData = (
  data: Record<string, any>,
  columnType: BaseMenuEnum,
  syncSchema: ISchema
): Record<string, any> => {
  const keys = []
  // schema原始对象
  function generateKey(obj) {
    if ('properties' in obj) {
      return generateKey(obj.properties)
    }
    for (const key in obj) {
      if (Object.prototype.hasOwnProperty.call(obj, key)) {
        const s = obj[key]
        if ('properties' in s) {
          generateKey(s.properties)
        } else if (s['x-decorator'] && s['x-decorator'] === 'FormItem') {
          keys.push(key)
        }
      }
    }
  }
  generateKey(connectSchema(SchemaEnum[columnType], syncSchema))
  const formData = {}
  if ('ID' in data) {
    formData.ID = data.ID
  }
  keys.forEach(key => {
    if (key in data) {
      formData[key] = data?.[key]
    }
  })
  return formData
}

export const useAsyncDataSource = service => field => {
  field.loading = true
  service(field).then(
    action.bound(data => {
      field.dataSource = data
      field.loading = false
    })
  )
}

/** 加载账号类型 */
export const loadAccountTypeData = async () => {
  const { code = -1, data = [] } = await getAccountTypeList()
  if (code === consts.RET_CODE.SUCCESS) {
    return data.map(item => ({ label: item.name, value: item.value }))
  }
  return []
}

/** 加载项目类型 */
export const loadProjectTypeData = async () => {
  const { code = -1, data = [] } = await getProjectTypeList()
  if (code === consts.RET_CODE.SUCCESS) {
    return data.map(item => ({ label: item.name, value: item.ID }))
  }
  return []
}

/** 加载组织架构 */
export const loadOrganizationalStructureList = async ({ dataID }) => {
  if (!dataID) return []
  const { code = -1, data = [] } = await queryOrganizationalStructureList({
    dataID,
    structureType: '1'
  })
  if (code === consts.RET_CODE.SUCCESS) {
    return data
  }
  return []
}
